package labss.lang.il.model.impl;

import labss.lang.il.model.*;

/**
 * @author Riccardo Solmi
 */
public class ILLanguageFactory implements LanguageFactory {
	// Singleton
	private static ILLanguageFactory instance;
	
	public static ILLanguageFactory instance() {
		if (instance == null)
			instance = new ILLanguageFactory();
		return instance;
	}
	
	private ILLanguageFactory() {}
	
	public Statement createWhile(Expression exp, Statement stm) {
		return new WhileStatement(exp, stm);
	}
	
	public Statement createIf(Expression exp, Statement thenStm) {
		return new IfStatement(exp, thenStm);
	}
	
	public Statement createIf(Expression exp, Statement thenStm, Statement elseStm) {
		return new IfElseStatement(exp, thenStm, elseStm);
	}
	
	public Statement createAssignment(Expression var, Expression exp) {
		return new AssignmentStatement(var, exp);
	}
	
	public Statement createBlock(Statement[] stm) {
		return new BlockStatement(stm);
	}
	
	public Literal createLiteral(boolean val) {
		return val ? BooleanLiteral.TRUE : BooleanLiteral.FALSE;
	}
	
	public Literal createLiteral(int val) {
		return new IntegerLiteral(val);
	}
	
	public Expression createVariable(String name) {
		return new Variable(name);
	}
	
	public Expression createAddition(Expression exp1, Expression exp2) {
		return new AdditionExpression(exp1, exp2);
	}
	
	public Expression createSubtraction(Expression exp1, Expression exp2) {
		return new SubtractionExpression(exp1, exp2);
	}
	
	public Expression createMultiplication(Expression exp1, Expression exp2) {
		return new MultiplicationExpression(exp1, exp2);
	}
	
	public Expression createDivision(Expression exp1, Expression exp2) {
		return new DivisionExpression(exp1, exp2);
	}
	
	public Expression createAnd(Expression exp1, Expression exp2) {
		return new AndExpression(exp1, exp2);
	}
	
	public Expression createOr(Expression exp1, Expression exp2) {
		return new OrExpression(exp1, exp2);
	}
	
	public Expression createNot(Expression exp) {
		return new NotExpression(exp);
	}
	
	public Expression createEquals(Expression exp1, Expression exp2) {
		return new EqualsExpression(exp1, exp2);
	}
	
	public Expression createLessThan(Expression exp1, Expression exp2) {
		return new LessThanExpression(exp1, exp2);
	}
	
	public Expression createGreaterThan(Expression exp1, Expression exp2) {
		return new GreaterThanExpression(exp1, exp2);
	}
}